@@ -23,7 +23,6 @@ import zipfile
 from typing import Iterator, Optional, Union
 
 import requests
-from cached_property import cached_property
 from logzero import setup_logger
 from PIL import Image
 from retry import retry
@@ -86,7 +85,7 @@ class BaseDevice():
             self._usbmux = usbmux
 
         self._udid = udid
-        self._info: DeviceInfo = self.info
+        self._info: DeviceInfo = None
         self._lock = threading.Lock()
         self._pair_record = None
 
@@ -104,21 +103,25 @@ class BaseDevice():
     def usbmux(self) -> Usbmux:
         return self._usbmux
 
-    @cached_property
+    @property
     def info(self) -> DeviceInfo:
+        if self._info:
+            return self._info
         devices = self._usbmux.device_list()
-        if not self._udid:
-            assert len(
-                devices
-            ) == 1, "Device is not present or multi devices connected"
-            _d = devices[0]
-            self._udid = _d.udid
-            return _d
-        else:
+        if self._udid:
             for d in devices:
                 if d.udid == self._udid:
-                    return d
-        raise MuxError("Device: {} not ready".format(self._udid))
+                    self._info = d
+        else:
+            if len(devices) != 1:
+                raise MuxError("Device is not present or multi devices connected")
+            _d = devices[0]
+            self._udid = _d.udid
+            self._info = _d
+
+        if not self._info:
+            raise MuxError("Device: {} not ready".format(self._udid))
+        return self._info
 
     def is_connected(self) -> bool:
         return self.udid in self.usbmux.device_udid_list()
@@ -129,7 +132,7 @@ class BaseDevice():
     
     @property
     def devid(self) -> int:
-        return self._info.device_id
+        return self.info.device_id
 
     @property
     def pair_record(self) -> dict:
@@ -275,7 +278,6 @@ class BaseDevice():
     def _system_BUID(self):
         return self.pair_record['SystemBUID']
 
-    @retry(OSError, tries=2, delay=3, logger=logger)
     def create_inner_connection(
             self,
             port: int = LOCKDOWN_PORT,  # 0xf27e,
@@ -297,14 +299,13 @@ class BaseDevice():
             'ProgName': PROGRAM_NAME,
         }
         logger.debug("Send payload: %s", payload)
-        with set_socket_timeout(conn.get_socket(), 5.0):
+        with set_socket_timeout(conn.get_socket, 5.0):
             data = conn.send_recv_packet(payload)
-        self._usbmux._check(data)
-        logger.debug("connected to port: %d", _port)
-
-        if _ssl:
-            conn.switch_to_ssl(self.ssl_pemfile_path)
-        return conn
+            self._usbmux._check(data)
+            logger.debug("connected to port: %d", _port)
+            if _ssl:
+                conn.switch_to_ssl(self.ssl_pemfile_path)
+            return conn
 
     @contextlib.contextmanager
     def create_session(self) -> PlistSocket:
@@ -485,19 +486,6 @@ class BaseDevice():
         copy_conn = self.start_service(LockdownService.CRASH_REPORT_COPY_MOBILE_SERVICE)
         return CrashManager(copy_conn)
 
-    def execute_crash_commands(self, output_dir, command):
-        move_conn = self.start_service(LockdownService.CRASH_REPORT_MOVER_SERVICE)
-        copy_conn = self.start_service(LockdownService.CRASH_REPORT_COPY_MOBILE_SERVICE)
-
-        cm = CrashManager(move_conn, copy_conn, output_dir)
-        if command == 'ls':
-            cm.preview()
-        elif command == 'cp':
-            cm.copy()
-        elif command == 'mv':
-            cm.move()
-        elif command == 'rm':
-            cm.remove()
     def start_service(self, name: str) -> PlistSocket:
         try:
             return self._unsafe_start_service(name)
@@ -515,10 +503,8 @@ class BaseDevice():
                 "Label": PROGRAM_NAME,
             })
             if 'Error' in data:  # data['Error'] is InvalidService
-                raise MuxServiceError("Could not start service: {0}! {2}".format(
-                    name, data['Error'],
-                    "Remember that you have to mount the Developer disk image on your device"
-                ))
+                error = data['Error'] # PasswordProtected, InvalidService
+                raise MuxServiceError(error)
 
         # Expect recv
         # {'EnableServiceSSL': True,
@@ -528,8 +514,9 @@ class BaseDevice():
         assert data['Service'] == name
         _ssl = data.get(
             'EnableServiceSSL',
-            False)  # and name != "com.apple.instruments.remoteserver"
+            False)
         conn = self.create_inner_connection(data['Port'], _ssl=_ssl)
+        conn.name = data['Service']
         return conn
 
     def screenshot(self) -> Image.Image:
@@ -571,7 +558,11 @@ class BaseDevice():
     @property
     def product_version(self) -> str:
         return self.get_value("ProductVersion", no_session=True)
-
+    
+    @property
+    def product_type(self) -> str:
+        return self.get_value("ProductType", no_session=True)
+    
     def app_sync(self, bundle_id: str, command: str = "VendDocuments") -> Sync:
         # Change command(VendContainer -> VendDocuments)
         # According to https://github.com/GNOME/gvfs/commit/b8ad223b1e2fbe0aec24baeec224a76d91f4ca2f
@@ -785,7 +776,7 @@ class BaseDevice():
         """
         return self.installation.uninstall(bundle_id)
 
-    def _connect_testmanagerd_lockdown(self):
+    def _connect_testmanagerd_lockdown(self) -> DTXService:
         if self.major_version() >= 14:
             conn = self.start_service(
                 LockdownService.TestmanagerdLockdownSecure)
@@ -817,7 +808,7 @@ class BaseDevice():
                     env: dict = {},
                     target_app_bundle_id: str = None,
                     logger: logging.Logger = logging,
-                    quit_event: threading.Event = None) -> int:  # pid
+                    quit_event: threading.Event = None) -> typing.Tuple[PlistSocket, int]:  # pid
 
         logger = logging.getLogger(LOG.xctest)
 
@@ -946,7 +937,7 @@ class BaseDevice():
         conn.register_callback(Event.NOTIFICATION, _callback)
         if quit_event:
             conn.register_callback(Event.FINISHED, lambda _: quit_event.set())
-        return pid
+        return conn, pid
 
     def major_version(self) -> int:
         version = self.get_value("ProductVersion")
@@ -968,22 +959,25 @@ class BaseDevice():
         return bundle_ids[0]
 
     def xctest(self, fuzzy_bundle_id="com.*.xctrunner", target_bundle_id=None, logger=None, env: dict={}):
+        """ Alias of xcuitest """
+        bundle_id = self._fnmatch_find_bundle_id(fuzzy_bundle_id)
+        logger.info("BundleID: %s", bundle_id)
+        return self.xcuitest(bundle_id, target_bundle_id=target_bundle_id, logger=logger, env=env)
+
+    def xcuitest(self, bundle_id, target_bundle_id=None, logger=None, env: dict={}):
         """
         Launch xctrunner and wait until quit
 
         Args:
+            bundle_id (str): xctrunner bundle id
             target_bundle_id (str): optional, launch WDA-UITests will not need it
             env: launch env
         """
         if not logger:
             logger = setup_logger(level=logging.INFO)
-        
-        bundle_id = self._fnmatch_find_bundle_id(fuzzy_bundle_id)
-        logger.info("BundleID: %s", bundle_id)
 
         product_version = self.get_value("ProductVersion")
         logger.info("ProductVersion: %s", product_version)
-
         logger.info("DeviceIdentifier: %s", self.udid)
 
         XCODE_VERSION = 29
@@ -1055,7 +1049,7 @@ class BaseDevice():
         # launch test app
         # index: 1540
         xclogger = setup_logger(name='xctest')
-        pid = self._launch_app_runner(bundle_id, session_identifier,
+        _, pid = self._launch_app_runner(bundle_id, session_identifier,
             target_app_bundle_id=target_bundle_id,
             env=env, logger=xclogger)
 
