@@ -11,10 +11,12 @@ import socket
 import ssl
 import struct
 import threading
+import typing
 import weakref
 from typing import Any, Union
 
 from ._proto import PROGRAM_NAME
+from ._utils import set_socket_timeout
 from .exceptions import *
 
 logger = logging.getLogger(PROGRAM_NAME)
@@ -37,6 +39,8 @@ class SafeStreamSocket:
         """
         self._id = get_uniq_id()
         self._sock = None
+        self._name = None
+
         if isinstance(addr, socket.socket):
             self._sock = addr
         else:
@@ -53,18 +57,22 @@ class SafeStreamSocket:
                 family = socket.AF_INET
             self._sock = socket.socket(family, socket.SOCK_STREAM)
             self._sock.connect(addr)
+        
+        self._sock_gclist = [self._sock]
 
-        self._finalizer = weakref.finalize(self, self._cleanup)
-    
-    def _cleanup(self):
-        logger.debug("Socket %d closed", self._id)
-        self._sock.close()
-        if hasattr(self, "_dup_sock"):
-            self._dup_sock.close()
+        def _cleanup(socks: typing.List[socket.socket]):
+            _id = str(self.id)
+            if self.name:
+                _id = self.name + ":" + str(self.id)
+            logger.debug("CLOSE(%s)", _id)
+            for sock in socks:
+                sock.close()
+
+        self._finalizer = weakref.finalize(self, _cleanup, self._sock_gclist)
     
     def close(self):
         self._finalizer()
-    
+        
     @property
     def closed(self) -> bool:
         return not self._finalizer.alive
@@ -73,6 +81,14 @@ class SafeStreamSocket:
     def id(self) -> int:
         return self._id
 
+    @property
+    def name(self) -> str:
+        return self._name
+    
+    @name.setter
+    def name(self, new_name: str):
+        self._name = new_name
+
     def get_socket(self) -> socket.socket:
         return self._sock
 
@@ -89,13 +105,15 @@ class SafeStreamSocket:
         return buf
 
     def sendall(self, data: Union[bytes, bytearray]) -> int:
-        return self._sock.sendall(data)
+        with set_socket_timeout(self._sock, 10):
+            return self._sock.sendall(data)
 
     def switch_to_ssl(self, pemfile):
         """ wrap socket to SSLSocket """
         # logger.debug("Switch to ssl")
         assert os.path.isfile(pemfile)
         self._dup_sock = self._sock.dup()
+        self._sock_gclist.append(self._dup_sock)
         
         # https://docs.python.org/zh-cn/3/library/ssl.html#ssl.SSLContext
         context = ssl.SSLContext(ssl.PROTOCOL_TLS)
@@ -189,6 +207,7 @@ class PlistSocketProperty:
     def __init__(self, psock: PlistSocket):
         self._psock = psock
         self.prepare()
+        self._finalizer = weakref.finalize(self, self.psock.close)
     
     @property
     def psock(self) -> PlistSocket:
@@ -203,4 +222,11 @@ class PlistSocketProperty:
     def recv_packet(self, header_size=None) -> dict:
         return self.psock.recv_packet(header_size)
     
+    def close(self):
+        self._finalizer()
+    
+    @property
+    def closed(self) -> bool:
+        return not self._finalizer.alive
+    
     
\ No newline at end of file
